@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 28050 $
+ * $Revision: 28262 $
  * $Author: rjongbloed $
- * $Date: 2012-07-18 02:15:30 -0500 (Wed, 18 Jul 2012) $
+ * $Date: 2012-08-28 01:20:50 -0500 (Tue, 28 Aug 2012) $
  */
 
 #include <ptlib.h>
@@ -1176,6 +1176,9 @@ PBoolean PArgList::Parse(const char * spec, PBoolean optionsBeforeParams)
 
   PINDEX codeCount = 0;
   while (*spec != '\0') {
+    while (*spec != '\0' && isspace(*spec))
+      ++spec;
+
     if (*spec == '-')
       optionLetters += ' ';
     else {
@@ -1190,11 +1193,11 @@ PBoolean PArgList::Parse(const char * spec, PBoolean optionsBeforeParams)
       PString newOpt(base, spec-base);
       PAssert(optionNames.GetValuesIndex(newOpt) == P_MAX_INDEX, "Multiple occurrences of same option string");
       optionNames[codeCount] = newOpt;
-      if (*spec == '.')
-        spec++;
     }
 
-    if (*spec == ':' || *spec == ';') {
+    if (*spec == '.')
+      spec++;
+    else if (*spec == ':' || *spec == ';') {
       canHaveOptionString.SetSize(codeCount+1);
       canHaveOptionString[codeCount] = *spec == ':' ? 2 : 1;
       spec++;
