@@ -30,9 +30,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25791 $
+ * $Revision: 29384 $
  * $Author: ededu $
- * $Date: 2011-05-17 14:04:01 -0500 (Tue, 17 May 2011) $
+ * $Date: 2013-03-28 22:58:24 -0500 (Thu, 28 Mar 2013) $
  */
 
 
@@ -40,9 +40,11 @@
 
 #ifdef P_DIRECTSHOW
 
+#include <set>
 #include <ptlib/videoio.h>
 #include <ptlib/vconvert.h>
 
+#include <ptlib/msos/ptlib/pt_atl.h>
 
 #ifdef _WIN32_WCE
 
@@ -86,18 +88,20 @@
 
 #else // _WIN32_WCE
 
+  /* workaround a compile error with mingw-w64 on sprintf member function
+     below. Even though the member function is not the same thing as the
+     global function which _is_ deprecated.
+
+     Also applies to a warning in MSVC.
+
+     Need to define this before dshow.h inclusion.
+   */
+  #define STRSAFE_NO_DEPRECATE
+
   #include <dshow.h>
   #include <initguid.h>
-
-#ifdef __MINGW32__
   #include <ks.h>
   #include <ksmedia.h>
-#else
-  #pragma warning(disable:4201)
-  #include <Ks.h>
-  #include <KsMedia.h>
-  #pragma warning(default:4201)
-#endif
 
   class PVideoInputControl_DirectShow : public PVideoInputControl
   {
@@ -175,49 +179,6 @@
 #endif
 
 
-//////////////////////////////////////////////////////////////////////
-
-    
-template <class T> class PComPtr
-{
-    T * pointer;
-  public:
-    PComPtr() : pointer(NULL) { }
-    ~PComPtr() { Release(); }
-
-    PComPtr & operator=(T * p)
-    {
-      Release();
-      if (p != NULL)
-        p->AddRef();
-      pointer = p;
-      return *this;
-    }
-
-    operator T *()        const { return  pointer; }
-    T & operator*()       const { return *pointer; }
-    T** operator&()             { return &pointer; }
-    T* operator->()       const { return  pointer; }
-    bool operator!()      const { return  pointer == NULL; }
-    bool operator<(T* p)  const { return  pointer < p; }
-    bool operator==(T* p) const { return  pointer == p; }
-    bool operator!=(T* p) const { return  pointer != p; }
-
-    void Release()
-    {
-      T * p = pointer;
-      if (p != NULL) {
-        pointer = NULL;
-        p->Release();
-      }
-    }
-
-  private:
-    PComPtr(const PComPtr &) {}
-    void operator=(const PComPtr &) { }
-};
-
-
 //////////////////////////////////////////////////////////////////////
 // Video Input device
 
@@ -269,26 +230,27 @@ class PVideoInputDevice_DirectShow : public PVideoInputDevice
     bool PlatformOpen();
     PINDEX GetCurrentBufferSize();
     bool GetCurrentBufferData(BYTE * data);
-    bool SetPinFormat();
+    bool SetPinFormat(unsigned useDefaultColourOrSize = 0);
     bool SetControlCommon(long control, int newValue);
     int GetControlCommon(long control);
 
 
     // DirectShow 
-    PComPtr<IGraphBuilder>         m_pGraphBuilder;
-    PComPtr<ICaptureGraphBuilder2> m_pCaptureBuilder;
-    PComPtr<IBaseFilter>           m_pCaptureFilter;
-    PComPtr<IPin>                  m_pCameraOutPin; // Camera output out -> Transform Input pin
+    CComPtr<IGraphBuilder>         m_pGraphBuilder;
+    CComPtr<ICaptureGraphBuilder2> m_pCaptureBuilder;
+    CComPtr<IBaseFilter>           m_pCaptureFilter;
+    CComPtr<IPin>                  m_pCameraOutPin; // Camera output out -> Transform Input pin
+    GUID                           m_selectedGUID;
 
 #ifdef _WIN32_WCE
-    PComPtr<PSampleGrabber>        m_pSampleGrabber;
+    PSampleGrabber               * m_pSampleGrabber;
 #else
-    PComPtr<ISampleGrabber>        m_pSampleGrabber;
-    PComPtr<ISampleGrabberCB>      m_pSampleGrabberCB;
-    PComPtr<IAMCameraControl>      m_pCameraControls;
+    CComPtr<ISampleGrabber>        m_pSampleGrabber;
+    CComPtr<ISampleGrabberCB>      m_pSampleGrabberCB;
+    CComPtr<IAMCameraControl>      m_pCameraControls;
 #endif
-    PComPtr<IBaseFilter>           m_pNullRenderer;
-    PComPtr<IMediaControl>         m_pMediaControl;
+    CComPtr<IBaseFilter>           m_pNullRenderer;
+    CComPtr<IMediaControl>         m_pMediaControl;
 
     PINDEX     m_maxFrameBytes;
     PBYTEArray m_tempFrame;
@@ -321,15 +283,15 @@ static struct {
     { "UYVY422", MEDIASUBTYPE_UYVY   }
 };
 
-static PString GUID2Format(GUID m_guid)
+static PString GUID2Format(GUID guid)
 {
    for (int j = 0; j < sizeof(ColourFormat2GUID)/sizeof(ColourFormat2GUID[0]); j++) {
-    if (m_guid == ColourFormat2GUID[j].m_guid)
+    if (guid == ColourFormat2GUID[j].m_guid)
       return ColourFormat2GUID[j].m_colourFormat;
    }
 
    wchar_t guidName[256];
-   if (StringFromGUID2(m_guid, guidName, sizeof(guidName)) <= 0)
+   if (StringFromGUID2(guid, guidName, sizeof(guidName)) <= 0)
        return "UNKNOWN"; // Can't use this entry!
 
    return guidName;
@@ -359,7 +321,6 @@ static PString ErrorMessage(HRESULT hr)
     return msg;
 #endif
 
-#pragma warning(disable:4995)
   char hex[20];
   _snprintf(hex, sizeof(hex), "0x%08x", hr);
   return hex;
@@ -431,83 +392,19 @@ class MediaTypePtr
     void operator=(const MediaTypePtr &) { }
 };
 
-//////////////////////////////////////////////////////////////////////////////////////////////////
-// Bubble Sorting functions
-// This is used to sort the video frame sizes from largest to smallest (removing duplicates)
-// The optimum frame size can be selected by sorting down the list for a given bandwidth.
-// If there is a better way to do this then great but it very important in order the framesizes. - SH
-
-class inputframes {
- public:
-    inputframes(unsigned W, unsigned H, unsigned R, PString C)
-        : width(W), height(H), rate(R), colour(C)
-    {};
-    unsigned width; unsigned height; unsigned rate; PString colour;
-};
-
-class frameSort {
-  public:
-     bool operator()(inputframes* const& f1, inputframes* const& f2) {
-          if (f1->width > f2->width) return true;
-          else if (f1->width == f2->width && f1->height > f2->height)
-              return true;
-          else return false;
-     }
-};
-
-class frameMatch {
-  public:
-     bool operator()(inputframes*& f1, inputframes*& f2) {
-          return (f1->width == f2->width) && (f1->height == f2->height);
-     }
-};
-
-class frameSizes : public std::vector<inputframes*>
-{
-  public:
-     ~frameSizes()
-     {
-          for (iterator pItem=begin(); pItem != end(); ++pItem)
-               delete *pItem;
-     }
-
-     void ReIndex() {
-         sort(begin(), end(), frameSort());
-         unique(begin(), end(), frameMatch());
-
-         unsigned lastWidth = P_MAX_INDEX;
-         int pos = 0;
-         iterator r = begin();
-         while (r != end()) {
-            const inputframes* frame = *r;
-            if (frame->width > lastWidth)  break;
-            else lastWidth = frame->width;
-            ++pos; ++r;
-         }
-
-         int sz = size();
-         PINDEX i = sz-1;
-         while (i > pos-1) {
-           erase(begin()+i);
-           --i;
-         }
-     }
-};
-
 
 //////////////////////////////////////////////////////////////////////////////////////////////////
 // Input device
 
 PVideoInputDevice_DirectShow::PVideoInputDevice_DirectShow()
   : m_maxFrameBytes(0)
+#ifdef _WIN32_WCE
+  , m_pSampleGrabber(NULL)
+#endif
 {
   PTRACE(4, "DShow\tVideo Device Instance");
 
   CoInitializeEx(NULL,COINIT_MULTITHREADED);
-
-  // Most common
-  preferredColourFormat = colourFormat = "BGR24";  
-  frameRate = 25;
 }
 
 
@@ -534,7 +431,7 @@ PBoolean PVideoInputDevice_DirectShow::GetDeviceCapabilities(const PString & dev
 
 bool PVideoInputDevice_DirectShow::GetDeviceCapabilities(Capabilities * caps) const
 {
-  PComPtr<IAMStreamConfig> pStreamConfig;
+  CComPtr<IAMStreamConfig> pStreamConfig;
 #ifdef __MINGW32__
   CHECK_ERROR_RETURN(m_pCameraOutPin->QueryInterface(IID_IAMStreamConfig, (void**)&pStreamConfig));
 #else
@@ -551,7 +448,8 @@ bool PVideoInputDevice_DirectShow::GetDeviceCapabilities(Capabilities * caps) co
     return false;
   }
 
-  frameSizes fsizes;
+  // Set is ordered large to small, and unique
+  std::set<PVideoFrameInfo, std::greater<PVideoFrameInfo> > fsizes;
   for (int iFormat = 0; iFormat < iCount; iFormat++) {
     MediaTypePtr pMediaFormat;
     if (SUCCEEDED(pStreamConfig->GetStreamCaps(iFormat, &pMediaFormat, (BYTE *)&scc)) &&
@@ -559,29 +457,19 @@ bool PVideoInputDevice_DirectShow::GetDeviceCapabilities(Capabilities * caps) co
         pMediaFormat->formattype == FORMAT_VideoInfo &&
         pMediaFormat->pbFormat != NULL &&
         pMediaFormat->cbFormat >= sizeof(VIDEOINFOHEADER))
-      fsizes.push_back(new inputframes(scc.MaxOutputSize.cx,
-                                       scc.MaxOutputSize.cy,
-                                       10000000/(unsigned)scc.MinFrameInterval,
-                                       GUID2Format(pMediaFormat->subtype)));
+      fsizes.insert(PVideoFrameInfo(scc.MaxOutputSize.cx,
+                                    scc.MaxOutputSize.cy,
+                                    GUID2Format(pMediaFormat->subtype),
+                                    10000000/(unsigned)scc.MinFrameInterval));
   }
 
-  if (fsizes.empty()) return false;
+  if (fsizes.empty())
+    return false;
 
   // Sort so we have unique sizes from largest to smallest
-  fsizes.ReIndex();
-  PINDEX i=0;
-  for (std::vector<inputframes*>::iterator r = fsizes.begin(); r != fsizes.end(); ++r) {
-    inputframes* f = *r;
-    PVideoFrameInfo cap;
-    cap.SetFrameSize(f->width,f->height);
-    cap.SetFrameRate(f->rate);
-    cap.SetColourFormat(f->colour);
-
-    PTRACE(5, "DShow\tFormat["<< i << "] = (" << cap.GetColourFormat() << ", "
-      << cap.GetFrameWidth() << "x" << cap.GetFrameHeight() << ", " << cap.GetFrameRate() << "fps)");
-
-    caps->framesizes.push_back(cap);
-    i++;
+  for (std::set<PVideoFrameInfo, std::greater<PVideoFrameInfo> >::iterator it = fsizes.begin(); it != fsizes.end(); ++it) {
+    PTRACE(5, "DShow\tFormat["<< caps->framesizes.size() << "] = " << *it);
+    caps->framesizes.push_back(*it);
   }
 
 #ifndef _WIN32_WCE
@@ -624,14 +512,14 @@ bool PVideoInputDevice_DirectShow::GetDeviceCapabilities(Capabilities * caps) co
 }
 
 
-bool PVideoInputDevice_DirectShow::SetPinFormat()
+bool PVideoInputDevice_DirectShow::SetPinFormat(unsigned useDefaultColourOrSize)
 {
   if (m_pCameraOutPin == NULL) {
     PTRACE(2, "DShow\tCamera output pin is NULL!");
     return false;
   }
 
-  PComPtr<IAMStreamConfig> pStreamConfig;
+  CComPtr<IAMStreamConfig> pStreamConfig;
 #ifdef __MINGW32__
   CHECK_ERROR_RETURN(m_pCameraOutPin->QueryInterface(IID_IAMStreamConfig, (void**)&pStreamConfig));
 #else
@@ -652,9 +540,17 @@ bool PVideoInputDevice_DirectShow::SetPinFormat()
         pMediaFormat->formattype == FORMAT_VideoInfo &&
         pMediaFormat->pbFormat != NULL &&
         pMediaFormat->cbFormat >= sizeof(VIDEOINFOHEADER) &&
-        scc.MaxOutputSize.cx == (LONG)frameWidth &&
-        scc.MaxOutputSize.cy == (LONG)frameHeight &&
-        GUID2Format(pMediaFormat->subtype) == colourFormat) {
+        (
+          useDefaultColourOrSize >= 2 ||
+          (
+            scc.MaxOutputSize.cx == (LONG)frameWidth &&
+            scc.MaxOutputSize.cy == (LONG)frameHeight &&
+            (
+              useDefaultColourOrSize >= 1 ||
+              GUID2Format(pMediaFormat->subtype) == colourFormat
+            )
+          )
+        )) {
 
       bool running = IsCapturing();
       if (running)
@@ -664,6 +560,17 @@ bool PVideoInputDevice_DirectShow::SetPinFormat()
       pVih->AvgTimePerFrame = 10000000 / frameRate;
       CHECK_ERROR_RETURN(pStreamConfig->SetFormat(pMediaFormat));
 
+      if (useDefaultColourOrSize >= 1) {
+        colourFormat = GUID2Format(pMediaFormat->subtype);
+        if (useDefaultColourOrSize >= 2) {
+          frameWidth = scc.MaxOutputSize.cx;
+          frameHeight = scc.MaxOutputSize.cy;
+        }
+      }
+
+      m_selectedGUID = pMediaFormat->subtype;
+      m_maxFrameBytes = CalculateFrameBytes(frameWidth, frameHeight, colourFormat);
+
       if (running)
         CHECK_ERROR_RETURN(m_pMediaControl->Run());
 
@@ -692,14 +599,14 @@ PBoolean PVideoInputDevice_DirectShow::Open(const PString & devName,
                                       NULL,
                                       CLSCTX_INPROC_SERVER,
                                       IID_IGraphBuilder,
-                                      (void **)&m_pGraphBuilder));
+                                      (LPVOID *)&m_pGraphBuilder));
 
   // Create the capture graph builder
   CHECK_ERROR_RETURN(CoCreateInstance(CLSID_CaptureGraphBuilder2,
                                       NULL,
                                       CLSCTX_INPROC_SERVER,
                                       IID_ICaptureGraphBuilder2,
-                                      (void **)&m_pCaptureBuilder));
+                                      (LPVOID *)&m_pCaptureBuilder));
 
   // Bind the Camera Input
   if (!BindCaptureDevice(devName))
@@ -715,14 +622,14 @@ PBoolean PVideoInputDevice_DirectShow::Open(const PString & devName,
   CHECK_ERROR_RETURN(m_pGraphBuilder->QueryInterface(IID_IMediaControl, (void **)&m_pMediaControl));
 
   // Get the camera output Pin 
-  PComPtr<IEnumPins> pEnum;
+  CComPtr<IEnumPins> pEnum;
   CHECK_ERROR_RETURN(m_pCaptureFilter->EnumPins(&pEnum));
 
   CHECK_ERROR_RETURN(pEnum->Reset());
   CHECK_ERROR_RETURN(pEnum->Next(1, &m_pCameraOutPin, NULL));
 
   // Set the format of the Output pin of the camera
-  if (!SetPinFormat())
+  if (!(SetPinFormat(0) || SetPinFormat(1) || SetPinFormat(2)))
     return false;
 
   if (!PlatformOpen())
@@ -755,7 +662,10 @@ PBoolean PVideoInputDevice_DirectShow::Close()
 
   // Release filters
 #ifdef _WIN32_WCE
-  delete m_pSampleGrabber;
+  if (m_pSampleGrabber != NULL) {
+    m_pSampleGrabber->Release();
+    delete m_pSampleGrabber;
+  }
 #else
   m_pNullRenderer.Release();
   m_pSampleGrabberCB.Release();
@@ -935,7 +845,7 @@ PBoolean PVideoInputDevice_DirectShow::GetFrameDataNoDelay(BYTE * destFrame, PIN
 
 int PVideoInputDevice_DirectShow::GetControlCommon(long control)
 {
-  PComPtr<IAMVideoProcAmp> pVideoProcAmp;
+  CComPtr<IAMVideoProcAmp> pVideoProcAmp;
   CHECK_ERROR(m_pCaptureFilter->QueryInterface(IID_IAMVideoProcAmp, (void **)&pVideoProcAmp), return -1);
 
   long minimum, maximum, stepping, def, flags;
@@ -995,7 +905,7 @@ PBoolean PVideoInputDevice_DirectShow::SetControlCommon(long control, int newVal
 {
   PTRACE(4, "DShow\tSetControl() = " << newValue);
 
-  PComPtr<IAMVideoProcAmp> pVideoProcAmp;
+  CComPtr<IAMVideoProcAmp> pVideoProcAmp;
   CHECK_ERROR_RETURN(m_pCaptureFilter->QueryInterface(IID_IAMVideoProcAmp, (void **)&pVideoProcAmp));
 
   long minimum, maximum, stepping, def, flags;
@@ -1227,9 +1137,9 @@ bool PVideoInputDevice_DirectShow::BindCaptureDevice(const PString & devName)
                                       NULL,
                                       CLSCTX_INPROC,
                                       IID_IBaseFilter,
-                                      (void**)&m_pCaptureFilter));
+                                      (LPVOID *)&m_pCaptureFilter));
 
-  PComPtr<IPersistPropertyBag> pPropertyBag;
+  CComPtr<IPersistPropertyBag> pPropertyBag;
   CHECK_ERROR_RETURN(m_pCaptureFilter->QueryInterface(&pPropertyBag));
 
   VARIANT varName;
@@ -1259,10 +1169,10 @@ bool PVideoInputDevice_DirectShow::PlatformOpen()
   CHECK_ERROR_RETURN(m_pGraphBuilder->AddFilter(dynamic_cast<IBaseFilter *>(grabber), L"Sampler"));
 
   // Find the source's output pin and the renderer's input pin
-  PComPtr<IPin> pCapturePinOut;
+  CComPtr<IPin> pCapturePinOut;
   CHECK_ERROR_RETURN(m_pCaptureFilter->FindPin(L"Capture", &pCapturePinOut));
 
-  PComPtr<IPin> pGrabberPinIn;
+  CComPtr<IPin> pGrabberPinIn;
   CHECK_ERROR_RETURN(m_pSampleGrabber->FindPin(L"In", &pGrabberPinIn));
 
   // Connect these two filters pins
@@ -1281,14 +1191,18 @@ bool PVideoInputDevice_DirectShow::PlatformOpen()
 
 class PComEnumerator
 {
-    PComPtr<ICreateDevEnum> m_pDevEnum;
-    PComPtr<IEnumMoniker>   m_pClassEnum;
-    PComPtr<IMoniker>       m_pMoniker;
+    CComPtr<ICreateDevEnum> m_pDevEnum;
+    CComPtr<IEnumMoniker>   m_pClassEnum;
+    CComPtr<IMoniker>       m_pMoniker;
   public:
     PComEnumerator()
     {
       // Create the system device enumerator
-      CHECK_ERROR(CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC, IID_ICreateDevEnum, (void **)&m_pDevEnum), return);
+      CHECK_ERROR(CoCreateInstance(CLSID_SystemDeviceEnum,
+                                   NULL,
+                                   CLSCTX_INPROC,
+                                   IID_ICreateDevEnum,
+                                   (LPVOID *)&m_pDevEnum), return);
       // Create an enumerator for the video capture devices
       CHECK_ERROR(m_pDevEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory, &m_pClassEnum, 0), return);
 
@@ -1310,7 +1224,7 @@ class PComEnumerator
 
     PString GetMonikerName()
     {
-      PComPtr<IPropertyBag> pPropBag;
+      CComPtr<IPropertyBag> pPropBag;
       CHECK_ERROR(m_pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pPropBag), return PString::Empty());
 
       // Find the description or friendly name.
@@ -1656,20 +1570,21 @@ bool PVideoInputDevice_DirectShow::PlatformOpen()
   // Buid the Camera Sample Grabber
   PTRACE(5, "DShow\tBuilding Sample Grabber");
 
-  PComPtr<IBaseFilter> pGrab; 
-  CHECK_ERROR_RETURN(CoCreateInstance(CLSID_SampleGrabber , NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void **)&pGrab));
+  CComPtr<IBaseFilter> pGrab; 
+  CHECK_ERROR_RETURN(CoCreateInstance(CLSID_SampleGrabber,
+                                      NULL,
+                                      CLSCTX_INPROC_SERVER,
+                                      IID_IBaseFilter,
+                                      (LPVOID *)&pGrab));
   CHECK_ERROR_RETURN(m_pGraphBuilder->AddFilter(pGrab, L"Sample Grabber"));
   CHECK_ERROR_RETURN(pGrab->QueryInterface(IID_ISampleGrabber, (void **)&m_pSampleGrabber));
 
   AM_MEDIA_TYPE mt;
   ZeroMemory(&mt, sizeof(AM_MEDIA_TYPE));
   mt.majortype = MEDIATYPE_Video;
-  mt.subtype = MEDIASUBTYPE_RGB24;
+  mt.subtype = m_selectedGUID;
   CHECK_ERROR_RETURN(m_pSampleGrabber->SetMediaType(&mt));
 
-  m_maxFrameBytes = CalculateFrameBytes(frameWidth, frameHeight, "RGB24");
-  PTRACE(5, "DShow\tBuffer size set to " << m_maxFrameBytes);
-
   CHECK_ERROR_RETURN(m_pSampleGrabber->SetBufferSamples(true));
 
   PTRACE(5, "DShow\tSetting Sample Grabber Callback");
@@ -1681,7 +1596,11 @@ bool PVideoInputDevice_DirectShow::PlatformOpen()
 
   // Set the NULL Renderer
   PTRACE(5, "DShow\tBuilding NULL output filter");
-  CHECK_ERROR_RETURN(CoCreateInstance (CLSID_NullRenderer , NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void **) &m_pNullRenderer));
+  CHECK_ERROR_RETURN(CoCreateInstance(CLSID_NullRenderer,
+                                      NULL,
+                                      CLSCTX_INPROC_SERVER,
+                                      IID_IBaseFilter,
+                                      (LPVOID *)&m_pNullRenderer));
 
   CHECK_ERROR_RETURN(m_pGraphBuilder->AddFilter(m_pNullRenderer, L"NULL Output"));
 
@@ -1731,7 +1650,8 @@ bool PVideoInputDevice_DirectShow::GetCurrentBufferData(BYTE * data)
 {
   CSampleGrabberCB * cb = (CSampleGrabberCB *)&*m_pSampleGrabberCB;
 
-  if (!cb->frameready.Wait(2000)) {
+  // Live! Cam Optia AF (VC0100) webcam took 3.1 sec.
+  if (!cb->frameready.Wait(5000)) {
     PTRACE(1, "DShow\tTimeout awaiting next frame");
     return false;
   }
